<?php
namespace FiloBlu\Eva\Block\Widget;

use Magento\CatalogWidget\Block\Product\ProductsList;

/**
 * Class ProductSlider
 * @package FiloBlu\Eva\Block\Widget
 */
class ProductSlider extends \Magento\Framework\View\Element\Template implements \Magento\Widget\Block\BlockInterface
{

    /**
     * @var \Magento\Catalog\Model\CategoryRepository
     */
    protected $_categoryRepository;
    /**
     * @var \Magento\Cms\Model\Template\FilterProvider
     */
    protected $_filterProvider;
    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product
     */
    protected $_resourceModelProduct;
    /**
     * @var \Magento\Catalog\Model\ProductRepository
     */
    protected $_productRepository;
    /**
     * @var \Magento\Cms\Helper\Page
     */
    protected $_cmsPageHelper;
    /**
     * @var \Magento\Cms\Model\PageFactory
     */
    protected $_pageFactory;
    /**
     * @var \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory
     */
    protected $_categoryCollectionFactory;
    /**
     * @var string
     */
    protected $_template = "widget/category-slider.phtml";


    /**
     * ProductSlider constructor.
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryCollectionFactory
     * @param \Magento\Catalog\Model\CategoryRepository $categoryRepository
     * @param \Magento\Cms\Model\Template\FilterProvider $filterProvider
     * @param \Magento\Catalog\Model\ResourceModel\Product $resourceModelProduct
     * @param \Magento\Catalog\Model\ProductRepository $productRepository
     * @param \Magento\Cms\Helper\Page $cmsPageHelper
     * @param \Magento\Cms\Model\PageFactory $pageFactory
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryCollectionFactory,
        \Magento\Catalog\Model\CategoryRepository $categoryRepository,
        \Magento\Cms\Model\Template\FilterProvider $filterProvider,
        \Magento\Catalog\Model\ResourceModel\Product $resourceModelProduct,
        \Magento\Catalog\Model\ProductRepository $productRepository,
        \Magento\Cms\Helper\Page $cmsPageHelper,
        \Magento\Cms\Model\PageFactory $pageFactory,
        \Psr\Log\LoggerInterface $logger
    ) {
        $this->_categoryCollectionFactory = $categoryCollectionFactory;
        parent::__construct($context);
        $this->_categoryRepository = $categoryRepository;
        $this->_filterProvider = $filterProvider;
        $this->_resourceModelProduct = $resourceModelProduct;
        $this->_productRepository = $productRepository;
        $this->_cmsPageHelper = $cmsPageHelper;
        $this->_pageFactory = $pageFactory;
        $this->logger = $logger;
    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getProductCollection()
    {
        $idPathCatalog = $this->getCategory();

        $idPath = explode('/', $idPathCatalog ?? '');

        $maxCollectionItem = 4;

        if (is_countable($this->getMaxitems()) && count($this->getMaxitems())) {
            $maxCollectionItem = $this->getMaxitems();
        };

        try {
            if (isset($idPath[0]) && isset($idPath[1]) && $idPath[0] == 'category') {
                $categoryId = $idPath[1];
                if ($categoryId) {
                    $category =  $this->_categoryCollectionFactory->create()->addAttributeToFilter("entity_id" , $categoryId)->getFirstItem();
                    $collection = $category->getProductCollection()->addAttributeToSelect('*');
                    $collection->getSelect()->limit($maxCollectionItem);
                    $collection->setOrder('position', 'ASC');
                    return $collection;
                }
            }
        }  catch (Exception $e) {
            $this->logger->critical($e->getMessage());
        }

    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getCategoryImage()
    {
        $idPathCatalog = $this->getCategory();
        $idPath = explode('/', $idPathCatalog ?? '');

        $categoryId = $idPath[1] ?? '';
        $category = $this->_categoryCollectionFactory->create()->addAttributeToSelect('*')->addAttributeToFilter("entity_id" , $categoryId)->getFirstItem();
        $categoryImageUrl = $category->getImageUrl();

        return $categoryImageUrl;
    }

    /**
     * @param $string
     * @return mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getFilteredContent($string) {
        return $this->_filterProvider->getBlockFilter()->setStoreId($this->_storeManager->getStore()->getId())->filter($string);
    }

    /**
     * @param $categoryId
     * @return |null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCategoryUrlById($categoryId) {
        return ( $categoryId ) ? $this->_categoryRepository->get($categoryId)->getUrl() : null;
    }

    /**
     * @param $productId
     * @return mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getProductUrlById($productId)
    {
        $product = $this->_productRepository->getById($productId);
        return $product->getUrlModel()->getUrl($product);
    }

    /**
     * @param $pageId
     * @return string
     */
    public function getCmsPageUrl($pageId) {
        return $this->_cmsPageHelper->getPageUrl($pageId);
    }

    /**
     * @return string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getWidgetUrl() {
        $_url = null;
        $linkType = $this->getLinkType();
        $customUri = $this->getLinkCustom();
        $categoryId = str_replace('category/','',$this->getLinkCategory() ?? '');
        $productId = str_replace('product/','',$this->getLinkProduct() ?? '');
        $pageId = $this->getLinkCmspage();

        switch ($linkType) {
            case 'default':
            case 1:
                $_url = '';
                break;

            case 2:
                $_url = $this->getCategoryUrlById($categoryId);
                break;

            case 3:
                $_url = $this->getProductUrlById($productId);
                break;

            case 4:
                $_url = $this->getCmsPageUrl($pageId);
                break;

            case 5:
                $_url = $this->getFilteredContent($customUri);
                break;
        }

        return trim($_url ?? '');
    }

    /**
     * @param $categoryId
     * @return string|null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCategoryTitle($categoryId) {
        return ( $categoryId ) ? $this->_categoryRepository->get($categoryId)->getName() : null;
    }

    /**
     * @param $productId
     * @return string|null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getProductName($productId)
    {
        $product = $this->_productRepository->getById($productId);
        return $product->getName();
    }

    /**
     * @param $pageId
     * @return string
     */
    public function getCmsPageTitle($pageId) {
        return $this->_pageFactory->create()->load($pageId)->getTitle();
    }
}
